@@ -174,7 +174,8 @@ impl<C: Context> Highway<C> {
         info!(%validators, instance=%instance_id, "creating Highway instance");
         let weights = validators.iter().map(Validator::weight);
         let banned = validators.iter_banned_idx();
-        let state = State::new(weights, params, banned);
+        let cannot_propose = validators.iter_cannot_propose_idx();
+        let state = State::new(weights, params, banned, cannot_propose);
         Highway {
             instance_id,
             validators,
@@ -247,7 +248,7 @@ impl<C: Context> Highway<C> {
     /// Returns the next missing dependency, or `None` if all dependencies of `pvv` are satisfied.
     ///
     /// If this returns `None`, `validate_vertex` can be called.
-    pub(crate) fn missing_dependency(&self, pvv: &PreValidatedVertex<C>) -> Option<Dependency<C>> {
+    pub(super) fn missing_dependency(&self, pvv: &PreValidatedVertex<C>) -> Option<Dependency<C>> {
         match pvv.inner() {
             Vertex::Evidence(_) | Vertex::Ping(_) => None,
             Vertex::Endorsements(endorsements) => {
