@@ -13,8 +13,6 @@ use fake_instant::FakeClock as Instant;
 use tracing::{debug, error, warn};
 
 use super::Config;
-#[cfg(test)]
-use super::Error;
 use crate::{types::NodeId, utils::DisplayIter};
 
 #[derive(Debug, PartialEq, Eq)]
@@ -31,6 +29,9 @@ pub(crate) enum GossipAction {
     ShouldGossip(ShouldGossip),
     /// We hold the data locally, and we shouldn't gossip the ID onwards.
     Noop,
+    /// We just finished gossiping the data: no need to gossip further, but an announcement that we
+    /// have finished gossiping this data should be made.
+    AnnounceFinished,
 }
 
 impl Display for GossipAction {
@@ -42,6 +43,7 @@ impl Display for GossipAction {
             GossipAction::AwaitingRemainder => write!(formatter, "awaiting remainder"),
             GossipAction::ShouldGossip(should_gossip) => Display::fmt(should_gossip, formatter),
             GossipAction::Noop => write!(formatter, "should do nothing"),
+            GossipAction::AnnounceFinished => write!(formatter, "finished gossiping"),
         }
     }
 }
@@ -176,13 +178,7 @@ pub(crate) struct GossipTable<T> {
     /// Data IDs for which gossiping is complete.
     finished: HashSet<T>,
     /// Timeouts for removal of items from the `finished` cache.
-    finished_timeouts: Timeouts<T>,
-    /// Data IDs for which gossiping has been paused (likely due to detecting that the data was not
-    /// correct as per our current knowledge).  Such data could later be decided as still requiring
-    /// to be gossiped, so we retain the `State` part here in order to resume gossiping.
-    paused: HashMap<T, State>,
-    /// Timeouts for removal of items from the `paused` cache.
-    paused_timeouts: Timeouts<T>,
+    timeouts: Timeouts<T>,
     /// See `Config::infection_target`.
     infection_target: usize,
     /// Derived from `Config::saturation_limit_percent` - we gossip data while the number of
@@ -202,11 +198,6 @@ impl<T> GossipTable<T> {
     pub fn items_finished(&self) -> usize {
         self.finished.len()
     }
-
-    /// Number of items for which gossipping is currently paused.
-    pub fn items_paused(&self) -> usize {
-        self.paused.len()
-    }
 }
 
 impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
@@ -217,9 +208,7 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
         GossipTable {
             current: HashMap::new(),
             finished: HashSet::new(),
-            finished_timeouts: Timeouts::new(),
-            paused: HashMap::new(),
-            paused_timeouts: Timeouts::new(),
+            timeouts: Timeouts::new(),
             infection_target: usize::from(config.infection_target()),
             holders_limit,
             finished_entry_duration: Duration::from_secs(config.finished_entry_duration_secs()),
@@ -251,12 +240,7 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
             return action;
         }
 
-        if let Some(action) = self.update_paused(data_id, update) {
-            debug!(item=%data_id, %action, "gossiping item is paused");
-            return action;
-        }
-
-        // This isn't in finished, current, or paused - add a new entry to current.
+        // This isn't in finished or current - add a new entry to current.
         let mut state = State::default();
         update(&mut state);
         let is_new = true;
@@ -278,12 +262,12 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
         &mut self,
         data_id: &T,
         maybe_holder: Option<NodeId>,
-    ) -> Option<ShouldGossip> {
+    ) -> GossipAction {
         self.purge_finished();
 
         if self.finished.contains(data_id) {
             debug!(item=%data_id, "no further action: item already finished");
-            return None;
+            return GossipAction::Noop;
         }
 
         let update = |state: &mut State| {
@@ -291,33 +275,19 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
             state.held_by_us = true;
         };
 
-        // Converts the returned action to an optional `ShouldGossip`.
-        let convert_action = |action: GossipAction| match action {
-            GossipAction::ShouldGossip(should_gossip) => Some(should_gossip),
-            GossipAction::Noop => None,
-            GossipAction::GetRemainder { .. } | GossipAction::AwaitingRemainder => {
-                unreachable!("can't be waiting for remainder since we hold the complete data")
-            }
-        };
-
         if let Some(action) = self.update_current(data_id, update) {
             debug!(item=%data_id, %action, "item is currently being gossiped");
-            return convert_action(action);
+            return action;
         }
 
-        if let Some(action) = self.update_paused(data_id, update) {
-            debug!(item=%data_id, %action, "gossiping item is paused");
-            return convert_action(action);
-        }
-
-        // This isn't in finished, current, or paused - add a new entry to current.
+        // This isn't in finished or current - add a new entry to current.
         let mut state = State::default();
         update(&mut state);
         let is_new = true;
         let action = state.action(self.infection_target, self.holders_limit, is_new);
         let _ = self.current.insert(*data_id, state);
         debug!(item=%data_id, %action, "gossiping new item should begin");
-        convert_action(action)
+        action
     }
 
     /// We got a response from a peer we gossiped to indicating we infected it (it didn't previously
@@ -358,29 +328,37 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
             state.in_flight_count = state.in_flight_count.saturating_sub(1);
         };
 
-        if let Some(action) = self.update_current(data_id, update) {
-            return action;
-        }
-
-        self.update_paused(data_id, update)
+        self.update_current(data_id, update)
             .unwrap_or(GossipAction::Noop)
     }
 
     /// Directly reduces the in-flight count of gossip requests for the given item by the given
     /// amount.
     ///
+    /// Returns `true` if there was a current entry for this data and it is now finished.
+    ///
     /// This should be called if, after trying to gossip to a given number of peers, we find that
     /// we've not been able to select enough peers.  Without this reduction, the given gossip item
-    /// would never move from `current` to `finished` or `paused`, and hence would never be purged.
-    pub(crate) fn reduce_in_flight_count(&mut self, data_id: &T, reduce_by: usize) {
-        if let Some(state) = self.current.get_mut(data_id) {
+    /// would never move from `current` to `finished`, and hence would never be purged.
+    pub(crate) fn reduce_in_flight_count(&mut self, data_id: &T, reduce_by: usize) -> bool {
+        let should_finish = if let Some(state) = self.current.get_mut(data_id) {
             state.in_flight_count = state.in_flight_count.saturating_sub(reduce_by);
             debug!(
                 item=%data_id,
                 in_flight_count=%state.in_flight_count,
                 "reduced in-flight count for item"
             );
+            state.in_flight_count == 0
+        } else {
+            false
+        };
+
+        if should_finish {
+            debug!(item=%data_id, "finished gossiping since no more peers to gossip to");
+            return self.force_finish(data_id);
         }
+
+        false
     }
 
     /// Checks if gossip request we sent timed out.
@@ -408,11 +386,7 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
             }
         };
 
-        if let Some(action) = self.update_current(data_id, update) {
-            return action;
-        }
-
-        self.update_paused(data_id, update)
+        self.update_current(data_id, update)
             .unwrap_or(GossipAction::Noop)
     }
 
@@ -431,8 +405,8 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
                 let _ = state.holders.remove(&peer);
                 debug!(item=%data_id, %peer, "removed peer as a holder of the item");
                 if state.holders.is_empty() {
-                    // We don't hold the full data, and we don't know any holders - pause the entry
-                    debug!(item=%data_id, "no further action: item now paused as no holders");
+                    // We don't hold the full data, and we don't know any holders - remove the entry
+                    debug!(item=%data_id, "no further action: item now removed as no holders");
                     return GossipAction::Noop;
                 }
             }
@@ -443,38 +417,19 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
             return action;
         }
 
-        if let Some(state) = self.paused.get_mut(data_id) {
-            if !state.held_by_us {
-                let _ = state.holders.remove(&peer);
-                debug!(item=%data_id, %peer, "removed peer as a holder of the item");
-            }
-        }
-
         GossipAction::Noop
     }
 
-    /// We have deemed the data not suitable for gossiping further.  If left in paused state, the
-    /// entry will eventually be purged, as for finished entries.
-    pub(crate) fn pause(&mut self, data_id: &T) {
-        if let Some(mut state) = self.current.remove(data_id) {
-            state.in_flight_count = 0;
-            let timeout = Instant::now() + self.finished_entry_duration;
-            let _ = self.paused.insert(*data_id, state);
-            let _ = self.paused_timeouts.push(timeout, *data_id);
-        }
-    }
-
-    /// Resumes gossiping of paused entry.
+    /// We have deemed the data not suitable for gossiping further.  The entry will be marked as
+    /// `finished` and eventually be purged.
     ///
-    /// Returns an error if gossiping this data is not in a paused state.
-    // TODO - remove lint relaxation once the method is used.
-    #[cfg(test)]
-    pub(crate) fn resume(&mut self, data_id: &T) -> Result<GossipAction, Error> {
-        let mut state = self.paused.remove(data_id).ok_or(Error::NotPaused)?;
-        let is_new = !state.held_by_us;
-        let action = state.action(self.infection_target, self.holders_limit, is_new);
-        let _ = self.current.insert(*data_id, state);
-        Ok(action)
+    /// Returns `true` if there was a current entry for this data.
+    pub(crate) fn force_finish(&mut self, data_id: &T) -> bool {
+        if self.current.remove(data_id).is_some() {
+            self.insert_to_finished(data_id);
+            return true;
+        }
+        false
     }
 
     /// Updates the entry under `data_id` in `self.current` and returns the action we should now
@@ -489,10 +444,8 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
         let mut state = self.current.remove(data_id)?;
         update(&mut state);
         if state.is_finished(self.infection_target, self.holders_limit) {
-            let timeout = Instant::now() + self.finished_entry_duration;
-            let _ = self.finished.insert(*data_id);
-            let _ = self.finished_timeouts.push(timeout, *data_id);
-            return Some(GossipAction::Noop);
+            self.insert_to_finished(data_id);
+            return Some(GossipAction::AnnounceFinished);
         }
         let is_new = false;
         let action = state.action(self.infection_target, self.holders_limit, is_new);
@@ -500,35 +453,19 @@ impl<T: Copy + Eq + Hash + Display> GossipTable<T> {
         Some(action)
     }
 
-    /// Updates the entry under `data_id` in `self.paused` and returns the action we should now
-    /// take, or `None` if the entry does not exist.
-    ///
-    /// If the entry becomes finished, it is moved from `self.paused` to `self.finished`.
-    fn update_paused<F: Fn(&mut State)>(&mut self, data_id: &T, update: F) -> Option<GossipAction> {
-        let mut state = self.paused.remove(data_id)?;
-        update(&mut state);
-        if state.is_finished(self.infection_target, self.holders_limit) {
-            let timeout = Instant::now() + self.finished_entry_duration;
-            let _ = self.finished.insert(*data_id);
-            let _ = self.finished_timeouts.push(timeout, *data_id);
-        } else {
-            let _ = self.paused.insert(*data_id, state);
-        }
-
-        Some(GossipAction::Noop)
+    fn insert_to_finished(&mut self, data_id: &T) {
+        let timeout = Instant::now() + self.finished_entry_duration;
+        let _ = self.finished.insert(*data_id);
+        let _ = self.timeouts.push(timeout, *data_id);
     }
 
     /// Retains only those finished entries which still haven't timed out.
     fn purge_finished(&mut self) {
         let now = Instant::now();
 
-        for expired_finished in self.finished_timeouts.purge(&now) {
+        for expired_finished in self.timeouts.purge(&now) {
             let _ = self.finished.remove(&expired_finished);
         }
-
-        for expired_paused in self.paused_timeouts.purge(&now) {
-            let _ = self.paused.remove(&expired_paused);
-        }
     }
 }
 
@@ -540,7 +477,9 @@ mod tests {
     use test::Bencher;
 
     use super::{super::config::DEFAULT_FINISHED_ENTRY_DURATION_SECS, *};
-    use crate::{crypto::hash::Digest, testing::TestRng, types::DeployHash, utils::DisplayIter};
+    use crate::{
+        crypto::hash::Digest, logging, testing::TestRng, types::DeployHash, utils::DisplayIter,
+    };
 
     const EXPECTED_DEFAULT_INFECTION_TARGET: usize = 3;
     const EXPECTED_DEFAULT_HOLDERS_LIMIT: usize = 15;
@@ -556,7 +495,6 @@ mod tests {
         let actual: BTreeSet<_> = gossip_table
             .current
             .get(data_id)
-            .or_else(|| gossip_table.paused.get(data_id))
             .map_or_else(BTreeSet::new, |state| state.holders.iter().collect());
         assert!(
             expected == actual,
@@ -568,6 +506,7 @@ mod tests {
 
     #[test]
     fn new_partial_data() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -598,25 +537,11 @@ mod tests {
         assert_eq!(GossipAction::AwaitingRemainder, action);
         check_holders(&node_ids[..2], &gossip_table, &data_id);
 
-        // Pause gossiping and check same partial data from third source causes `Noop` to be
-        // returned and holders updated.
-        gossip_table.pause(&data_id);
-        let action = gossip_table.new_partial_data(&data_id, node_ids[2]);
-        assert_eq!(GossipAction::Noop, action);
-        check_holders(&node_ids[..3], &gossip_table, &data_id);
-
-        // Reset the data and check same partial data from fourth source causes `AwaitingRemainder`
-        // to be returned and holders updated.
-        gossip_table.resume(&data_id).unwrap();
-        let action = gossip_table.new_partial_data(&data_id, node_ids[3]);
-        assert_eq!(GossipAction::AwaitingRemainder, action);
-        check_holders(&node_ids[..4], &gossip_table, &data_id);
-
         // Finish the gossip by reporting three infections, then check same partial data causes
         // `Noop` to be returned and holders cleared.
         let _ = gossip_table.new_complete_data(&data_id, Some(node_ids[0]));
-        let limit = 4 + EXPECTED_DEFAULT_INFECTION_TARGET;
-        for node_id in &node_ids[4..limit] {
+        let limit = 3 + EXPECTED_DEFAULT_INFECTION_TARGET;
+        for node_id in &node_ids[3..limit] {
             let _ = gossip_table.we_infected(&data_id, *node_id);
         }
         let action = gossip_table.new_partial_data(&data_id, node_ids[limit]);
@@ -636,6 +561,7 @@ mod tests {
 
     #[test]
     fn should_noop_if_we_have_partial_data_and_get_gossip_response() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_id = NodeId::random(&mut rng);
         let data_id: u64 = rng.gen();
@@ -653,6 +579,7 @@ mod tests {
 
     #[test]
     fn new_complete_data() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -661,7 +588,7 @@ mod tests {
 
         // Check new complete data from us causes `ShouldGossip` to be returned.
         let action = gossip_table.new_complete_data(&data_id, None);
-        let expected = Some(ShouldGossip {
+        let expected = GossipAction::ShouldGossip(ShouldGossip {
             count: EXPECTED_DEFAULT_INFECTION_TARGET,
             exclude_peers: HashSet::new(),
             is_already_held: false,
@@ -672,7 +599,7 @@ mod tests {
         // Check same complete data from other source causes `Noop` to be returned since we still
         // have all gossip requests in flight.  Check it updates holders.
         let action = gossip_table.new_complete_data(&data_id, Some(node_ids[0]));
-        assert!(action.is_none());
+        assert_eq!(GossipAction::Noop, action);
         check_holders(&node_ids[..1], &gossip_table, &data_id);
 
         // Check receiving a gossip response, causes `ShouldGossip` to be returned and holders
@@ -686,42 +613,25 @@ mod tests {
         assert_eq!(expected, action);
         check_holders(&node_ids[..2], &gossip_table, &data_id);
 
-        // Pause gossiping and check same complete data from third source causes `Noop` to be
-        // returned and holders updated.
-        gossip_table.pause(&data_id);
         let action = gossip_table.new_complete_data(&data_id, Some(node_ids[2]));
-        assert!(action.is_none());
+        assert_eq!(GossipAction::Noop, action);
         check_holders(&node_ids[..3], &gossip_table, &data_id);
 
-        // Reset the data and check same complete data from fourth source causes Noop` to be
-        // returned since we still have all gossip requests in flight.  Check it updates holders.
-        let action = gossip_table.resume(&data_id).unwrap();
-        let expected = GossipAction::ShouldGossip(ShouldGossip {
-            count: EXPECTED_DEFAULT_INFECTION_TARGET,
-            exclude_peers: node_ids[..3].iter().cloned().collect(),
-            is_already_held: true,
-        });
-        assert_eq!(expected, action);
-
-        let action = gossip_table.new_complete_data(&data_id, Some(node_ids[3]));
-        assert!(action.is_none());
-        check_holders(&node_ids[..4], &gossip_table, &data_id);
-
         // Finish the gossip by reporting enough non-infections, then check same complete data
         // causes `Noop` to be returned and holders cleared.
-        let limit = 4 + EXPECTED_DEFAULT_INFECTION_TARGET;
-        for node_id in &node_ids[4..limit] {
+        let limit = 3 + EXPECTED_DEFAULT_INFECTION_TARGET;
+        for node_id in &node_ids[3..limit] {
             let _ = gossip_table.we_infected(&data_id, *node_id);
         }
         let action = gossip_table.new_complete_data(&data_id, None);
-        assert!(action.is_none());
+        assert_eq!(GossipAction::Noop, action);
         check_holders(&node_ids[..0], &gossip_table, &data_id);
 
         // Time the finished data out, then check same complete data causes `ShouldGossip` to be
         // returned as per a completely new entry.
         Instant::advance_time(DEFAULT_FINISHED_ENTRY_DURATION_SECS * 1_000 + 1);
         let action = gossip_table.new_complete_data(&data_id, Some(node_ids[0]));
-        let expected = Some(ShouldGossip {
+        let expected = GossipAction::ShouldGossip(ShouldGossip {
             count: EXPECTED_DEFAULT_INFECTION_TARGET,
             exclude_peers: node_ids[..1].iter().cloned().collect(),
             is_already_held: false,
@@ -732,6 +642,7 @@ mod tests {
 
     #[test]
     fn should_terminate_via_infection_limit() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -761,12 +672,13 @@ mod tests {
 
         // Check third new infection does cause us to stop gossiping.
         let action = gossip_table.we_infected(&data_id, node_ids[limit]);
-        assert_eq!(GossipAction::Noop, action);
+        assert_eq!(GossipAction::AnnounceFinished, action);
         assert!(gossip_table.finished.contains(&data_id));
     }
 
     #[test]
     fn should_terminate_via_incoming_gossip() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id1: u64 = rng.gen();
@@ -787,16 +699,20 @@ mod tests {
 
         // Simulate receiving a final gossip request for each, which should cause them both to be
         // moved to the `finished` collection.
-        let _ = gossip_table.new_partial_data(&data_id1, node_ids[EXPECTED_DEFAULT_HOLDERS_LIMIT]);
+        let action =
+            gossip_table.new_partial_data(&data_id1, node_ids[EXPECTED_DEFAULT_HOLDERS_LIMIT]);
         assert!(gossip_table.finished.contains(&data_id1));
+        assert_eq!(GossipAction::AnnounceFinished, action);
 
-        let _ = gossip_table
+        let action = gossip_table
             .new_complete_data(&data_id2, Some(node_ids[EXPECTED_DEFAULT_HOLDERS_LIMIT]));
         assert!(gossip_table.finished.contains(&data_id2));
+        assert_eq!(GossipAction::AnnounceFinished, action);
     }
 
     #[test]
     fn should_terminate_via_checking_timeout() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -813,12 +729,39 @@ mod tests {
 
         // Simulate a gossip response timing out, which should cause the item to be moved to the
         // `finished` collection.
-        let _ = gossip_table.check_timeout(&data_id, node_ids[EXPECTED_DEFAULT_HOLDERS_LIMIT]);
+        let action = gossip_table.check_timeout(&data_id, node_ids[EXPECTED_DEFAULT_HOLDERS_LIMIT]);
+        assert!(gossip_table.finished.contains(&data_id));
+        assert_eq!(GossipAction::AnnounceFinished, action);
+    }
+
+    #[test]
+    fn should_terminate_via_reducing_in_flight_count() {
+        let _ = logging::init();
+        let mut rng = crate::new_rng();
+        let data_id: u64 = rng.gen();
+
+        let mut gossip_table = GossipTable::new(Config::default());
+
+        // Take the item close to the termination condition of in-flight count reaching 0.  It
+        // should remain unfinished.
+        let _ = gossip_table.new_complete_data(&data_id, None);
+        let limit = EXPECTED_DEFAULT_INFECTION_TARGET - 1;
+        assert!(!gossip_table.reduce_in_flight_count(&data_id, limit));
+        assert!(!gossip_table.finished.contains(&data_id));
+
+        // Reduce the in-flight count to 0, which should cause the item to be moved to the
+        // `finished` collection.
+        assert!(gossip_table.reduce_in_flight_count(&data_id, 1));
+        assert!(gossip_table.finished.contains(&data_id));
+
+        // Check that calling this again has no effect and continues to return `false`.
+        assert!(!gossip_table.reduce_in_flight_count(&data_id, 1));
         assert!(gossip_table.finished.contains(&data_id));
     }
 
     #[test]
     fn should_terminate_via_saturation() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -851,11 +794,12 @@ mod tests {
 
         // Check 15th non-infection does cause us to stop gossiping.
         let action = gossip_table.we_infected(&data_id, node_ids[limit]);
-        assert_eq!(GossipAction::Noop, action);
+        assert_eq!(GossipAction::AnnounceFinished, action);
     }
 
     #[test]
     fn should_not_terminate_below_infection_limit_and_saturation() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -886,6 +830,7 @@ mod tests {
 
     #[test]
     fn check_timeout_should_detect_holder() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -916,6 +861,7 @@ mod tests {
         should_panic(expected = "shouldn't check timeout for a gossip response for partial data")
     )]
     fn check_timeout_should_panic_for_partial_copy() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -927,6 +873,7 @@ mod tests {
 
     #[test]
     fn should_remove_holder_if_unresponsive() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -950,39 +897,13 @@ mod tests {
         // and the entry should be removed since there are no more holders.
         let action = gossip_table.remove_holder_if_unresponsive(&data_id, node_ids[1]);
         assert_eq!(GossipAction::Noop, action);
-        check_holders(&node_ids[..0], &gossip_table, &data_id);
         assert!(!gossip_table.current.contains_key(&data_id));
-        assert!(!gossip_table.paused.contains_key(&data_id));
-
-        // Add new partial data from node 2 and check gossiping has been resumed.
-        let action = gossip_table.new_partial_data(&data_id, node_ids[2]);
-        let expected = GossipAction::GetRemainder {
-            holder: node_ids[2],
-        };
-        assert_eq!(expected, action);
-        check_holders(&node_ids[2..3], &gossip_table, &data_id);
-
-        // Node 2 should be removed from the holders since it hasn't provided us with the full data,
-        // and the entry should be paused since there are no more holders.
-        let action = gossip_table.remove_holder_if_unresponsive(&data_id, node_ids[2]);
-        assert_eq!(GossipAction::Noop, action);
-        check_holders(&node_ids[..0], &gossip_table, &data_id);
-        assert!(!gossip_table.current.contains_key(&data_id));
-        assert!(!gossip_table.paused.contains_key(&data_id));
-
-        // Add new complete data from node 3 and check gossiping has been resumed.
-        let action = gossip_table.new_complete_data(&data_id, Some(node_ids[3]));
-        let expected = Some(ShouldGossip {
-            count: EXPECTED_DEFAULT_INFECTION_TARGET,
-            exclude_peers: iter::once(node_ids[3]).collect(),
-            is_already_held: false,
-        });
-        assert_eq!(expected, action);
-        check_holders(&node_ids[3..4], &gossip_table, &data_id);
+        assert!(!gossip_table.finished.contains(&data_id));
     }
 
     #[test]
     fn should_not_remove_holder_if_responsive() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -998,35 +919,29 @@ mod tests {
         assert_eq!(GossipAction::Noop, action); // Noop as all RPCs are still in-flight
         check_holders(&node_ids[..1], &gossip_table, &data_id);
         assert!(gossip_table.current.contains_key(&data_id));
-        assert!(!gossip_table.paused.contains_key(&data_id));
     }
 
     #[test]
-    fn should_not_auto_resume_manually_paused() {
+    fn should_force_finish() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
 
         let mut gossip_table = GossipTable::new(Config::default());
 
-        // Add new partial data from node 0, manually pause gossiping, then record that node 0
-        // failed to provide the full data.
+        // Add new partial data from node 0, then forcibly finish gossiping.
         let _ = gossip_table.new_partial_data(&data_id, node_ids[0]);
-        gossip_table.pause(&data_id);
-        let action = gossip_table.remove_holder_if_unresponsive(&data_id, node_ids[0]);
-        assert_eq!(GossipAction::Noop, action);
-        check_holders(&node_ids[..0], &gossip_table, &data_id);
+        assert!(gossip_table.force_finish(&data_id));
+        assert!(gossip_table.finished.contains(&data_id));
 
-        // Add new partial data from node 1 and check gossiping has not been resumed.
-        let action = gossip_table.new_partial_data(&data_id, node_ids[1]);
-        assert_eq!(GossipAction::Noop, action);
-        check_holders(&node_ids[1..2], &gossip_table, &data_id);
-        assert!(!gossip_table.current.contains_key(&data_id));
-        assert!(gossip_table.paused.contains_key(&data_id));
+        // Ensure forcibly finishing the same data returns `false`.
+        assert!(!gossip_table.force_finish(&data_id));
     }
 
     #[test]
     fn should_purge() {
+        let _ = logging::init();
         let mut rng = crate::new_rng();
         let node_ids = random_node_ids(&mut rng);
         let data_id: u64 = rng.gen();
@@ -1045,15 +960,100 @@ mod tests {
         gossip_table.purge_finished();
         assert!(!gossip_table.finished.contains(&data_id));
 
-        // Add new complete data and pause.
+        // Add new complete data and forcibly finish.
         let _ = gossip_table.new_complete_data(&data_id, None);
-        gossip_table.pause(&data_id);
-        assert!(gossip_table.paused.contains_key(&data_id));
+        assert!(gossip_table.force_finish(&data_id));
+        assert!(gossip_table.finished.contains(&data_id));
 
-        // Time the paused data out and check it has been purged.
+        // Time the finished data out and check it has been purged.
         Instant::advance_time(DEFAULT_FINISHED_ENTRY_DURATION_SECS * 1_000 + 1);
         gossip_table.purge_finished();
-        assert!(!gossip_table.paused.contains_key(&data_id));
+        assert!(!gossip_table.finished.contains(&data_id));
+    }
+
+    #[test]
+    fn timeouts_purge_in_order() {
+        let mut timeouts = Timeouts::new();
+        let now = Instant::now();
+        let later_100 = now + Duration::from_millis(100);
+        let later_200 = now + Duration::from_millis(200);
+
+        // Timeouts are added and purged in chronological order.
+        timeouts.push(now, 0);
+        timeouts.push(later_100, 1);
+        timeouts.push(later_200, 2);
+
+        let now_after_time_travel = now + Duration::from_millis(10);
+        let purged = timeouts.purge(&now_after_time_travel).collect::<Vec<i32>>();
+
+        assert_eq!(purged, vec![0]);
+    }
+
+    #[test]
+    fn timeouts_depends_on_binary_search_by_implementation() {
+        // This test is meant to document the dependency of
+        // Timeouts::purge on https://doc.rust-lang.org/std/vec/struct.Vec.html#method.binary_search_by.
+        // If this test is failing then it's reasonable to believe that the implementation of
+        // binary_search_by has been updated.
+        let mut timeouts = Timeouts::new();
+        let now = Instant::now();
+        let later_100 = now + Duration::from_millis(100);
+        let later_200 = now + Duration::from_millis(200);
+        let later_300 = now + Duration::from_millis(300);
+        let later_400 = now + Duration::from_millis(400);
+        let later_500 = now + Duration::from_millis(500);
+        let later_600 = now + Duration::from_millis(600);
+
+        timeouts.push(later_100, 1);
+        timeouts.push(later_200, 2);
+        timeouts.push(later_300, 3);
+
+        // If a node's system time was changed to a time earlier than
+        // the earliest timeout, and a new timeout is added with an instant
+        // corresponding to this new early time, then this would make the earliest
+        // timeout the LAST timeout in the vec.
+        // [100 < 200 < 300 > 0]
+        timeouts.push(now, 0);
+
+        let now_after_time_travel = now + Duration::from_millis(10);
+        // Intuitively, we would expect [1,2,3,0] to be in the "purged" vec here.
+        // This is not the case because we're using binary_search_by, which (currently)
+        // is implemented with logic that checks if a, b, ... z are in a consistent order.
+        // in this case, the order that we've established is a < b < ... < z for each element in the
+        // vec, but we broke that order by inserting '0' last, and for some reason,
+        // binary_search_by won't find this unless there is a number > n occurring AFTER n
+        // in the vec.
+
+        let purged = timeouts.purge(&now_after_time_travel).collect::<Vec<i32>>();
+        let empty: Vec<i32> = vec![];
+
+        // This isn't a problem and the order will eventually
+        // be restored.
+        assert_eq!(purged, empty);
+
+        timeouts.push(later_400, 4);
+        timeouts.push(later_500, 5);
+        timeouts.push(later_600, 6);
+
+        // Now, we advance time another 10 ms and purge again.
+        // In this scenario, timeouts with a later time are added after our
+        // improperly ordered "now" timeout
+        // [100 < 200 < 300 > 0 < 400 < 500 < 600]
+        let now_after_time_travel = now + Duration::from_millis(20);
+        let purged = timeouts.purge(&now_after_time_travel).collect::<Vec<i32>>();
+        let expected = [1, 2, 3, 0];
+
+        assert_eq!(purged, expected);
+
+        // After the previous purge, an order is restored where a < b for consecutive elements in
+        // the vec. [400 < 500 < 600], so, purging timeouts up to 610 will properly clear
+        // the vec.
+        let now_after_time_travel = now + Duration::from_millis(610);
+        let purged = timeouts.purge(&now_after_time_travel).collect::<Vec<i32>>();
+        let expected = [4, 5, 6];
+
+        assert_eq!(purged, expected);
+        assert_eq!(0, timeouts.values.len());
     }
 
     #[bench]
